#########################################
######### Replication script ############
#########################################

#########################################
##### Standard joint frailty model ######
#########################################

library("frailtypack")
data("readmission", package = "frailtypack")

# Approximate computational time <1 minute
modJoint.gap <- frailtyPenal(Surv(time,event) ~ cluster(id) + dukes +
  charlson + sex + chemo + terminal(death),
  formula.terminalEvent = ~ dukes + charlson + sex + chemo,
  data = readmission, n.knots = 8, kappa = c(2.11e+08, 9.53e+11))
modJoint.gap

# Goodness-of-fit plots

pdf(file = "residuals1.pdf", 8,5 )
par(mfrow = c(1, 2))
plot(aggregate(readmission$t.stop, by = list(readmission$id),
  FUN = max)[2][, 1], modJoint.gap$martingale.res, ylab = "",
  xlab = "time", main = "Rehospitalizations", ylim = c(-4, 4))
lines(lowess(aggregate(readmission$t.stop, by = list(readmission$id),
  FUN = max)[2][, 1], modJoint.gap$martingale.res, f = 1), lwd = 3,
  col = "grey")
plot(aggregate(readmission$t.stop, by = list(readmission$id),
  FUN = max)[2][, 1], modJoint.gap$martingaledeath.res, ylab = "",
  xlab = "time", main = "Death", ylim = c(-2, 2))
lines(lowess(aggregate(readmission$t.stop, by = list(readmission$id),
  FUN = max)[2][, 1], modJoint.gap$martingaledeath.res, f = 1), lwd = 3,
  col = "grey")
dev.off()

# Individual predictions of the frailties
pdf(file = "frailty1.pdf", 8, 5)
par(mfrow = c(1, 1))
plot(1:403, modJoint.gap$frailty.pred, xlab = "Id of patients",
  ylab = "Frailty predictions for each patient", type = "p", axes = FALSE,
  cex = as.vector(table(readmission$id)), pch = 1, ylim = c(-0.1, 7),
  xlim = c(-2, 420))
axis(1, round(seq(0, 403, length = 10), digit = 0))
axis(2, round(seq(0, 7, length = 10), digit = 1))
dev.off()

#### Time-varying coefficients ####

# Approximate computational time 18 minutes
modJoint.gap.timedep <- frailtyPenal(Surv(time,event) ~ cluster(id) +
  dukes + charlson + timedep(sex) + chemo + terminal(death),
  formula.terminalEvent = ~ dukes + charlson + timedep(sex) + chemo,
  data = readmission, n.knots = 8, kappa = c(2.11e+08, 9.53e+11),
  betaorder = 3, betaknots = 3)
modJoint.gap.timedep

pdf(file = "plot_timedep_2311.pdf", 11, 8)
par(mfrow = c(1, 1))
modJoint.gap.timedep
dev.off()

# Likelihood ratio tests

LR.statistic <- -2 * modJoint.gap$logLik + 2 * modJoint.gap.timedep$logLik
p.value <- signif(1 - pchisq(LR.statistic, df = 10), 5) #10 dof: 2*(3+3-1)
p.value

# Model without variable sex

# Approximate computational time <1 minute
modJoint.gap.nosex <- frailtyPenal(Surv(time,event) ~ cluster(id) + dukes +
  charlson + chemo + terminal(death), formula.terminalEvent = ~ dukes +
  charlson + chemo, data = readmission, n.knots = 8, kappa = c(2.11e+08, 9.53e+11))
modJoint.gap.nosex

LR.statistic <- -2 * modJoint.gap.nosex$logLik + 2 * modJoint.gap.timedep$logLik
p.value <- signif(1 - pchisq(LR.statistic, df = 12), 5) # 12 dof: 2*(3+3)
p.value

#### Dynamic predictions ####

# Data preparation
datapred <- data.frame(time = 0, event = 0, id = 0, dukes = 0, charlson = 0,
  sex = 0, chemo = 0)
datapred[, 4:7] <- lapply(datapred[, 4:7], as.factor)
levels(datapred$dukes) <- c(1, 2, 3)
levels(datapred$charlson) <- c(1, 2, 3)
levels(datapred$sex) <- c(1, 2)
levels(datapred$chemo) <- c(1, 2)
datapred[1, ] <- c(365, 1, 1, 1, 1, 1, 2) #Patient 1
datapred[2, ] <- c(548, 1, 1, 1, 1, 1, 2) #Patient 1
datapred[3, ] <- c(365, 1, 2, 3, 1, 1, 2) #Patient 2
datapred[4, ] <- c(548, 1, 2, 3, 1, 1, 2) #Patient 2

# Predictions
pred <- prediction(modJoint.gap, datapred, t = 730, window = seq(1, 1096, 183),
  MC.sample = 500)

pdf(file = "prediction_modJointgap1911.pdf", 11, 8)
plot(pred, conf.bands = TRUE)
dev.off()

#####################################################
###### Joint frailty model for clustered data #######
#####################################################

# Data preparation 

readmission <- transform(readmission, group = id %% 6 + 1 )
readm_cluster <- subset(readmission, (t.start == 0 & event == 1 ) | event == 0)

# Model fit

# Approximate computational time <1 minute
joi.clus <- frailtyPenal(Surv(t.start, t.stop, event) ~ cluster(group) +
  num.id(id) + dukes + sex + chemo + terminal(death),
  formula.terminalEvent = ~ dukes + sex + chemo, data = readm_cluster,
  n.knots = 8, kappa = c(1.e+10, 1.e+10), recurrentAG = TRUE, Alpha = "None")
joi.clus

###########################################
####### General joint frailty model #######
###########################################

# Approximate computational time 1 minute
modJoint.general <- frailtyPenal(Surv(time, event) ~ cluster(id) + dukes +
  charlson + sex + chemo + terminal(death), formula.terminalEvent = ~ dukes +
  charlson + sex + chemo, data = readmission, jointGeneral = TRUE, n.knots = 8,
  kappa = c(2.11e+08, 9.53e+11))
modJoint.general

###########################################
###### Multivariate frailty model #########
###########################################

data("dataMultiv", package = "frailtypack")

# Approximate computational time 55 minutes
modMultiv.spli <- multivPenal(Surv(TIMEGAP, INDICREC) ~ cluster(PATIENT) +
  v1 + v2 + event2(INDICMETA) + terminal(INDICDEATH), formula.Event2 = ~ v1 +
  v2 + v3, formula.terminalEvent = ~ v1, data = dataMultiv,
  n.knots = c(8, 8, 8), kappa = c(1, 1, 1), initialize = FALSE)
modMultiv.spli

###################################################################
##### Joint model for longitudinal data and a terminal event ######
###################################################################

data("colorectalLongi", package = "frailtypack")
head(colorectalLongi, 10)

data("colorectal", package = "frailtypack")
head(colorectal, 5)

colorectalSurv <- subset(colorectal, new.lesions == 0) # information from recurrent events are not used

# Model with the link function: random effects

# Approximate computational time 2 minutes
modLongi <- longiPenal(Surv(time1, state) ~ age + treatment + who.PS
  + prev.resection, tumor.size ~  year * treatment  + age+who.PS ,
  colorectalSurv,data.Longi = colorectalLongi, random = c("1", "year"),
  id = "id", link = "Random-effects", left.censoring = -3.33,
  n.knots = 8, kappa = 0.93, method.GH = "Pseudo-adaptive", n.nodes = 7)
modLongi

par(mfrow = c(2, 2))
plot(modLongi, main = "Hazard function")
plot(aggregate(colorectalSurv$time1, by = list(colorectalSurv$id), 
     FUN = max)[2][, 1], modLongi$martingaledeath.res, ylab = "", xlab = 
     "time", main = "Martingale Residuals - Death", ylim = c(-4.2, 4.2))
lines(lowess(aggregate(colorectalSurv$time1, by = list(colorectalSurv$id), 
      FUN = max)[2][, 1], modLongi$martingaledeath.res, f = 1), lwd = 3, 
      col = "grey")
qqnorm(modLongi$marginal_chol.res, main = "Marginal Cholesky residuals", 
       xlab = "")
qqline(modLongi$marginal_chol.res)
plot(modLongi$pred.y.cond, modLongi$conditional.res, xlab = "Fitted",
     ylab = "Conditional residuals",
     main = "Conditional Residuals vs. Fitted Values")

# Model with the link function: current level 

# Approximate computational time 10 minutes
modLongi2 <- longiPenal(Surv(time1, state) ~ age + treatment +
  who.PS + prev.resection, tumor.size ~  year * treatment + age + who.PS, 
  colorectalSurv, data.Longi = colorectalLongi, random = c("1", "year"), 
  id = "id", link = "Current-level", left.censoring = -3.33, 
  n.knots = 8, kappa = 0.93, method.GH = "Pseudo-adaptive", n.nodes = 7)

# Comparison of models using EPOCE

pdf(file = "epoce_biv_1201.pdf", 11, 8)
par(mfrow = c(1, 2))
time <- seq(1.2, 2.4, 0.1)
epoce <- epoce(modLongi, time)
epoce2 <- epoce(modLongi2, time)
diff <- Diffepoce(epoce, epoce2)
plot(time, epoce$cvpol, ylab = "CVPOL", xlab = "Time", pch = 6, 
  col = "darkcyan", type= "b", ylim = c(0.4, 1.0))
points(time, epoce2$cvpol, pch = 15, col = 'brown3',  type = "b", lty = 2)
legend(1.9, 1, legend = c("modLongi", "modLongi2"), pch = c(6, 15), 
  col = c("darkcyan", "brown3"), bty = "n", cex = 0.7, lty = c(1, 2))
plot(diff)
dev.off()
epoce$AtRisk

#######################################################
##### Trivariate joint model for longitudinal data ####
##### recurrent events and a terminal event ###########
#######################################################

set.seed(1007)

# Parameter initialisation, reduced model (shared frailty model)
modShared <- frailtyPenal(Surv(time0, time1, new.lesions) ~ cluster(id) + age + 
  treatment + who.PS, data = colorectal, recurrentAG = TRUE,
  n.knots = 8, cross.validation = TRUE, kappa = 1000, RandDist = "LogN")

# Approximate computational time 43 minutes
modTrivariate <- trivPenal(Surv(time0, time1, new.lesions) ~ cluster(id) + 
  age + treatment + who.PS + terminal(state), 
  formula.terminalEvent = ~ age + treatment + who.PS + prev.resection, 
  tumor.size ~ year * treatment + age + who.PS, data = colorectal, 
  data.Longi = colorectalLongi, random = c("1", "year"), id = "id", 
  link = "Random-effects", left.censoring = -3.33, recurrentAG = TRUE,
  n.knots = 7, kappa = c(0.01, 0.7), method.GH = "Pseudo-adaptive", n.nodes = 7,
  init.B = c(-0.18, -0.22, -0.24, -0.22, 0.35, -0.23, -0.10, 
  -0.09, -0.12, 0.80, -0.23, 3.02, -0.30, 0.05, -0.02, -0.29, 0.11, 0.74, -0.63))

modTrivariate

# Goodness-of-fit plots

pdf(file = "diagnostic_tri_1201.pdf", 11, 8)
par(mfrow = c(2, 3))
plot(modTrivariate, main = "Hazard functions")
plot(modTrivariate, type = "Survival", main = "Survival functions")
plot(aggregate(colorectal$time1, by = list(colorectal$id), FUN = max)[2][, 1],
  modTrivariate$martingaledeath.res, ylab = "", xlab = "time", 
  main = "Martingale Residuals - Death", ylim = c(-4.2, 4.2))
lines(lowess(aggregate(colorectal$time1, by = list(colorectal$id),
  FUN = max)[2][, 1], modTrivariate$martingaledeath.res, f = 1), lwd = 3, 
  col = "grey")
plot(aggregate(colorectal$time1, by = list(colorectal$id), 
  FUN = max)[2][, 1], modTrivariate$martingale.res, ylab = "", xlab = "time", 
  main = "Martingale Residuals\n  - Recurrences", ylim = c(-4.2, 4.2))
lines(lowess(aggregate(colorectal$time1, by = list(colorectal$id),
  FUN = max)[2][, 1], modTrivariate$martingale.res, f = 1), lwd = 3,
  col = "grey")
qqnorm(modTrivariate$marginal_chol.res, main = "Marginal Cholesky 
  residuals" ,xlab = "")
qqline(modTrivariate$marginal_chol.res)
plot(modTrivariate$pred.y.cond, modTrivariate$conditional.res, 
  xlab = "Fitted", ylab = "Conditional residuals", main = "Conditional 
  Residuals \n vs. Fitted Values")
dev.off()

#### Dynamic predictions ####

# Prediction time and horizons definition
predtime <- 1.0
window <- seq(0.1, 1.5, 0.1)
fwindow <- predtime + window

# Predictions using the bivariate model
pred_bivariate <- prediction(modLongi, data = colorectalSurv, 
  data.Longi = colorectalLongi, predtime, window)

# Predictions using the trivariate model
pred_trivariate <- prediction(modTrivariate, data = colorectal, 
  data.Longi = colorectalLongi, predtime, window)

# Loading the packages and modified functions to calculate the Brier
# Score
library("pec")
library("prodlim")
source("ipcw_2-9_modif.R")       
source("pecMethods_2-9_modif.R") 

# Data format preparation
data_surv <- colorectalSurv[, c(1, 3, 9)]
predictions_bivariate <- as.matrix(pred_bivariate$pred)
survj_biv <- as.matrix(predictions_bivariate[data_surv$time1 >= predtime, ])
survj_biv <- 1 - cbind(0, survj_biv)
predictions_trivariate <- as.matrix(pred_trivariate$pred)
survj_tri <- as.matrix(predictions_trivariate[data_surv$time1 >= predtime, ])
survj_tri <- 1 - cbind(0, survj_tri)

# Prediction error calculation
BrierScore <- pec(list("Bivariate" = survj_biv, "Trivariate" = survj_tri),
                  formula = Surv(time1, state) ~ 1,
                  data = data_surv[data_surv$time1 >= predtime, ],
                  cens.model ="marginal", data.cens = data_surv, 
                  exact = FALSE, times = fwindow, ptime = predtime, reference = FALSE)

# Plotting the results

pdf(file = "BrierScore_1201.pdf", 11, 8)
par(mfrow = c(1, 1))
plot(fwindow, BrierScore$AppErr$Bivariate[-1], pch = 6, main="", 
  ylab = "Prediction error", xlab = "Years", ylim = c(0, 0.3), 
  xlim = c(0.9, 2.7), col = "black", type = "l", lwd = 2, axes = FALSE, lty = 1)
points(fwindow, BrierScore$AppErr$Trivariate[-1], pch = 15, col = "blue",
  type = "l", lwd = 2, lty = 2)
legend(2.2, 0.3, legend = c("modLongi", "modTrivariate"), bty = "n", 
  cex = 1.1, lty = c(1, 2), lwd = 2, col = c("black", "blue"))
axis(1, at = seq(0.9, 2.7, by = 0.3))
axis(2, at = seq(0, 0.3, by = 0.1))
abline(v = 1, lty = 2, lwd = 2, col = "gray35")
dev.off()

## Dynamic predictions for a joint model for a longitudinal biomarker
## and a terminal event

# Construction of the dataframe for predictions
datapredj_longi <- data.frame(id = 0, year = 0, tumor.size = 0, treatment = 0,
  age = 0, who.PS = 0)
datapredj_longi$treatment <- factor(datapredj_longi$treatment, levels = 1:2)
datapredj_longi$age <- factor(datapredj_longi$age, levels = 1:3)
datapredj_longi$who.PS <- factor(datapredj_longi$who.PS, levels = 1:3)

# Patient 1
datapredj_longi[1, ] <- c(1, 0, 1.2, 2, 1, 1)
datapredj_longi[2, ] <- c(1, 0.3, 1.1, 2, 1, 1)
datapredj_longi[3, ] <- c(1, 0.6, 1.4, 2, 1, 1)
datapredj_longi[4, ] <- c(1, 0.9, 2.2, 2, 1, 1)
datapredj_longi[5, ] <- c(1, 1.5, 3.0, 2, 1, 1)

# Patient 2
datapredj_longi[6, ] <- c(2, 0, 1.2, 2, 1, 1)
datapredj_longi[7, ] <- c(2, 0.3, 1.1, 2, 1, 1)
datapredj_longi[8, ] <- c(2, 0.5, 0.7, 2, 1, 1)
datapredj_longi[9, ] <- c(2, 0.7, 0.3, 2, 1, 1)
datapredj_longi[10, ] <- c(2, 0.9, 0.1, 2, 1, 1)

# Dataframe with covariates related to the terminal event
datapredj <- data.frame(id = 0, treatment = 0, age = 0, who.PS = 0, 
  prev.resection = 0)
datapredj$treatment <- factor(datapredj$treatment, levels = 1:2)
datapredj$age <- factor(datapredj$age, levels = 1:3)
datapredj$who.PS <- factor(datapredj$who.PS, levels = 1:3)
datapredj$prev.resection <- factor(datapredj$prev.resection, levels = 1:2)
datapredj[1, ] <- c(1, 2, 1, 1, 1)
datapredj[2, ] <- c(2, 2, 1, 1, 1)

# Prediction of death between 1 and 1+t given biomarker (bivariate
# model) (with confidence intervals)

pdf(file = "prediction_biv_tri_1201.pdf")
par(mfrow = c(1, 2))
pred.joint <- prediction(modLongi, datapredj, datapredj_longi, 1.0, 
  seq(0.5, 2.5, 0.2), MC.sample = 500)
plot(pred.joint, conf.bands = TRUE)

# Dataframe with history of recurrent event and covariates related to
# the recurrent and terminal events
datapredj <- data.frame(time0 = 0, time1 = 0, new.lesions = 0, id = 0, 
  treatment = 0, age = 0, who.PS = 0, prev.resection = 0)
datapredj$treatment <- factor(datapredj$treatment, levels = 1:2)
datapredj$age <- factor(datapredj$age, levels = 1:3)
datapredj$who.PS <- factor(datapredj$who.PS, levels = 1:3)
datapredj$prev.resection <- factor(datapredj$prev.resection, levels = 1:2)
datapredj[1, ] <- c(0, 0.4, 1, 1, 2, 1, 1, 1)
datapredj[2, ] <- c(0.4, 1.2, 1, 1, 2, 1, 1, 1)
datapredj[3, ] <- c(0, 0.5, 1, 2, 2, 1, 1, 1)

# Prediction of death between 0.5 and 0.5+t given biomarker and the
# recurrences (with confidence intervals)
pred.joint2 <- prediction(modTrivariate, datapredj, datapredj_longi, 1.0,
  seq(0.5, 2.5, 0.2), MC.sample = 500)
plot(pred.joint2, conf.bands = TRUE)
dev.off()

